{ *****************************************************
                    TCustomPopup Component

  A set of popup dialog components.

                  Paul Warren
         HomeGrown Software Development
       (c) 1997 Langley British Columbia.
                (604) 856-6523
         e-mail:  hg_soft@uniserve.com
    Home page: http://users.uniserve.com/~hg_soft
  ***************************************************** }

unit Popups;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, hgDate;

type
  TCustomPopup = class(TComponent)
  private
    { private declarations }
    FAlignSource: TWinControl;
    PopupPoint: TPoint;
    procedure SetAlignSource(Value: TWinControl);
  protected
    { protected declarations }
    procedure GetPopupPoint; virtual;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
  public
    { public declarations }
    function Execute(var Value): boolean; virtual; abstract;
  published
    { published declarations }
    property AlignSource: TWinControl read FAlignSource write SetAlignSource;
  end;

  TDatePopup = class(TCustomPopup)
  private
    { private declarations }
  protected
    { protected declarations }
  public
    { public declarations }
    constructor Create(AOwner: TComponent); override;
    function Execute(var Value): boolean; override;
  published
    { published declarations }
  end;

procedure Register;

implementation

{$IFDEF WIN32}
  {$R POPUPS.D32}
{$ELSE}
  {$R POPUPS.D16}
{$ENDIF}

procedure TCustomPopup.GetPopupPoint;
begin
  { Set the Popup's Top/Left point to the Bottom Left hand
    corner of the associated TWinControl }
  PopupPoint.Y := (AlignSource.Top + AlignSource.Height);
  PopupPoint.X := AlignSource.Left;
  PopupPoint := AlignSource.Parent.ClientToScreen(PopupPoint);
end;

procedure TCustomPopup.Notification(AComponent: TComponent;
      Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (AComponent = FAlignSource) then
     FAlignSource := nil;
end;

procedure TCustomPopup.SetAlignSource(Value: TWinControl);
begin
  FAlignSource := Value;
  GetPopupPoint;
end;

{ TDatePopup }
constructor TDatePopup.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
end;

function TDatePopup.Execute(var Value): boolean;
var
  ThePopup: TDateDlg;
begin
  { create the form }
  ThePopup := TDateDlg.Create(Application);
  ThePopup.PopupDate := TDateTime(Value);
  if (AlignSource <> nil) then
  begin
    GetPopupPoint;
    ThePopup.Top := PopupPoint.Y;
    ThePopup.Left := PopupPoint.X;
    if (GetSystemMetrics(SM_CYSCREEN) < (ThePopup.Top + ThePopup.Height)) then
      ThePopup.Top := (PopupPoint.Y - (ThePopup.Height + 2) - AlignSource.Height);
    if (GetSystemMetrics(SM_CXSCREEN) < (ThePopup.Left + ThePopup.Width)) then
      ThePopup.Left := (PopupPoint.X - (ThePopup.Width + 2) + AlignSource.Width);
  end;
  try
    if ThePopup.ShowModal <> mrCancel then
    begin
      TDateTime(Value) := ThePopup.PopupDate;
      Result := True;
    end else
      Result := False;
  finally
    ThePopup.Free;
  end;
end;

{ register component on HomeGrown page }
procedure Register;
begin
  RegisterComponents('HomeGrown', [TDatePopup]);
end;

end.